"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.M3uParser = void 0;
const m3u_playlist_1 = require("./m3u-playlist");
class M3uParser {
    /**
     *
     * @param attributesString tvg-id="" group-title=""
     * @private
     */
    static getAttributes(attributesString) {
        const attributeValuePair = attributesString.split('" ');
        const attributes = new m3u_playlist_1.M3uAttributes();
        attributeValuePair.forEach((item) => {
            const [key, value] = item.split('="');
            attributes[key] = value.replace('"', '');
        });
        return attributes;
    }
    /**
     *
     * @param trackInformation -1 tvg-id="" group-title="",Tv Name
     * @param media
     * @private
     */
    static processMedia(trackInformation, media) {
        const lastCommaIndex = trackInformation.lastIndexOf(',');
        const durationAttributes = trackInformation.substring(0, lastCommaIndex);
        media.name = trackInformation.substring(lastCommaIndex + 1);
        const firstSpaceIndex = durationAttributes.indexOf(' ');
        media.duration = Number(durationAttributes.substring(0, firstSpaceIndex));
        const attributes = durationAttributes.substring(firstSpaceIndex + 1);
        media.attributes = this.getAttributes(attributes);
    }
    /**
     *
     * @param item #EXTINF:-1 tvg-id="" group-title="",Tv Name
     * @param playlist
     * @param media
     * @private
     */
    static processDirective(item, playlist, media) {
        const firstSemicolonIndex = item.indexOf(':');
        const directive = item.substring(0, firstSemicolonIndex);
        const trackInformation = item.substring(firstSemicolonIndex + 1);
        switch (directive) {
            case m3u_playlist_1.M3uDirectives.EXTINF: {
                this.processMedia(trackInformation, media);
                break;
            }
            case m3u_playlist_1.M3uDirectives.EXTGRP: {
                media.group = trackInformation;
                break;
            }
            case m3u_playlist_1.M3uDirectives.PLAYLIST: {
                playlist.title = trackInformation;
                break;
            }
        }
    }
    static getPlaylist(lines) {
        const playlist = new m3u_playlist_1.M3uPlaylist();
        let media = new m3u_playlist_1.M3uMedia('');
        lines.forEach(item => {
            if (this.isDirective(item)) {
                this.processDirective(item, playlist, media);
            }
            else {
                media.location = item;
                playlist.medias.push(media);
                media = new m3u_playlist_1.M3uMedia('');
            }
        });
        return playlist;
    }
    static isDirective(item) {
        return item[0] === m3u_playlist_1.M3U_COMMENT;
    }
    static isValidM3u(firstLine) {
        return firstLine[0].startsWith(m3u_playlist_1.M3uDirectives.EXTM3U);
    }
    static parse(m3uString) {
        const lines = m3uString.split('\n').filter(item => item);
        if (!this.isValidM3u(lines)) {
            throw new Error(`Missing ${m3u_playlist_1.M3uDirectives.EXTM3U} directive!`);
        }
        return this.getPlaylist(lines);
    }
}
exports.M3uParser = M3uParser;
